package de.intevation.mxd.reader;

import java.io.IOException;
import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.IFillSymbol;
import com.esri.arcgis.display.SimpleFillSymbol;
import com.esri.arcgis.display.MultiLayerFillSymbol;
import com.esri.arcgis.display.esriSimpleFillStyle;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;

import org.w3c.dom.Element;

import de.intevation.mxd.utils.MapToXMLUtils;

/**
 * Reads simple marker symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class SimpleFillSymbolReader
implements ISymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(SimpleFillSymbolReader.class);

    /**
     * Private member.
     */
    private Element renderer;
    private SimpleFillSymbol symbol;
    private MapToXMLUtils util;


    public SimpleFillSymbolReader(ISymbol symbol) throws Exception{
        logger.debug("contructor(ISymbol)");
        if(symbol instanceof SimpleFillSymbol) {
            this.symbol = (SimpleFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a SimpleFillSymbol!");
        }
    }

    public SimpleFillSymbolReader(IFillSymbol symbol) throws Exception{
        logger.debug("contructor(IFillSymbol)");
        if(symbol instanceof SimpleFillSymbol) {
            this.symbol = (SimpleFillSymbol)symbol;
        }
        else {
            throw new Exception("Not a SimpleFillSymbol!");
        }
    }

    /**
     * Setter for the parent XML element.
     *
     * @param parent The XML parent node.
     */
    public void setParent(Element parent) {
        this.renderer = parent;
    }

    /**
     * Setter for XML document helper.
     *
     * @param util The helper class for storing map information.
     */
    public void setUtil(MapToXMLUtils util) {
        this.util = util;
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read()
    throws Exception {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(renderer);

        symbolElement.setAttribute("name", symbol.getNameString());
        symbolElement.setAttribute("style", "fill");
        int style = symbol.getStyle();
        switch(style) {
            case esriSimpleFillStyle.esriSFSCross:
                symbolElement.setAttribute("fillstyle", "cross"); break;
            case esriSimpleFillStyle.esriSFSSolid:
                symbolElement.setAttribute("fillstyle", "solid"); break;
            case esriSimpleFillStyle.esriSFSVertical:
                symbolElement.setAttribute("fillstyle", "vertical"); break;
            case esriSimpleFillStyle.esriSFSHorizontal:
                symbolElement.setAttribute("fillstyle", "horizontal"); break;
            case esriSimpleFillStyle.esriSFSForwardDiagonal:
                symbolElement.setAttribute("fillstyle", "fwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSBackwardDiagonal:
                symbolElement.setAttribute("fillstyle", "bwdiagonal"); break;
            case esriSimpleFillStyle.esriSFSDiagonalCross:
                symbolElement.setAttribute("fillstyle", "diagonalcross");
                break;
            default: symbolElement.setAttribute("fillstyle", "empty");
        }

        if(symbol.getColor() instanceof IRgbColor) {
            IRgbColor color = (IRgbColor)symbol.getColor();
            Color c = new Color (
                color.getRed(),
                color.getGreen(),
                color.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            symbolElement.setAttribute("transparency",
                String.valueOf(color.getTransparency()));
        }
        else {
            RgbColor col = new RgbColor();
            col.setRGB(symbol.getColor().getRGB());
            Color c = new Color (
                col.getRed(),
                col.getGreen(),
                col.getBlue());
            symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            symbolElement.setAttribute("transparency",
                String.valueOf(col.getTransparency()));
        }

        ILineSymbol ls = symbol.getOutline();
        LineSymbolReader lsr = new LineSymbolReader();
        if(lsr.canRead(ls)) {
            lsr.setSymbol(ls);
            lsr.setUtil(util);
            lsr.setParent(symbolElement);
            lsr.read();
        }
        else {
            logger.debug("The type of " + ls.getClass().toString() +
                         " is not implemented!");
        }

        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
